@@ -1,6 +1,6 @@
 #include "equalhash.h"
 
-fltype_t *get_type(value_t t)
+fltype_t *get_type(fl_context_t *fl_ctx, value_t t)
 {
     fltype_t *ft;
     if (issymbol(t)) {
@@ -8,24 +8,24 @@ fltype_t *get_type(value_t t)
         if (ft != NULL)
             return ft;
     }
-    void **bp = equalhash_bp(&TypeTable, (void*)t);
+    void **bp = equalhash_bp_r(&fl_ctx->TypeTable, (void*)t, (void*)fl_ctx);
     if (*bp != HT_NOTFOUND)
         return (fltype_t*)*bp;
 
-    int align, isarray=(iscons(t) && car_(t) == arraysym && iscons(cdr_(t)));
+    int align, isarray=(iscons(t) && car_(t) == fl_ctx->arraysym && iscons(cdr_(t)));
     size_t sz;
     if (isarray && !iscons(cdr_(cdr_(t)))) {
         // special case: incomplete array type
         sz = 0;
     }
     else {
-        sz = ctype_sizeof(t, &align);
+        sz = ctype_sizeof(fl_ctx, t, &align);
     }
 
     ft = (fltype_t*)malloc(sizeof(fltype_t));
     ft->type = t;
     if (issymbol(t)) {
-        ft->numtype = sym_to_numtype(t);
+        ft->numtype = sym_to_numtype(fl_ctx, t);
         ((symbol_t*)ptr(t))->type = ft;
     }
     else {
@@ -40,10 +40,10 @@ fltype_t *get_type(value_t t)
     ft->init = NULL;
     if (iscons(t)) {
         if (isarray) {
-            fltype_t *eltype = get_type(car_(cdr_(t)));
+            fltype_t *eltype = get_type(fl_ctx, car_(cdr_(t)));
             if (eltype->size == 0) {
                 free(ft);
-                lerror(ArgError, "invalid array element type");
+                lerror(fl_ctx, fl_ctx->ArgError, "invalid array element type");
             }
             ft->elsz = eltype->size;
             ft->eltype = eltype;
@@ -55,15 +55,15 @@ fltype_t *get_type(value_t t)
     return ft;
 }
 
-fltype_t *get_array_type(value_t eltype)
+fltype_t *get_array_type(fl_context_t *fl_ctx, value_t eltype)
 {
-    fltype_t *et = get_type(eltype);
+    fltype_t *et = get_type(fl_ctx, eltype);
     if (et->artype != NULL)
         return et->artype;
-    return get_type(fl_list2(arraysym, eltype));
+    return get_type(fl_ctx, fl_list2(fl_ctx, fl_ctx->arraysym, eltype));
 }
 
-fltype_t *define_opaque_type(value_t sym, size_t sz, cvtable_t *vtab,
+fltype_t *define_opaque_type(value_t sym, size_t sz, const cvtable_t *vtab,
                              cvinitfunc_t init)
 {
     fltype_t *ft = (fltype_t*)malloc(sizeof(fltype_t));
@@ -79,14 +79,14 @@ fltype_t *define_opaque_type(value_t sym, size_t sz, cvtable_t *vtab,
     return ft;
 }
 
-void relocate_typetable(void)
+void relocate_typetable(fl_context_t *fl_ctx)
 {
-    htable_t *h = &TypeTable;
+    htable_t *h = &fl_ctx->TypeTable;
     size_t i;
     void *nv;
     for(i=0; i < h->size; i+=2) {
         if (h->table[i] != HT_NOTFOUND) {
-            nv = (void*)relocate((value_t)h->table[i]);
+            nv = (void*)relocate(fl_ctx, (value_t)h->table[i]);
             h->table[i] = nv;
             if (h->table[i+1] != HT_NOTFOUND)
                 ((fltype_t*)h->table[i+1])->type = (value_t)nv;
