@@ -1,7 +1,7 @@
-// This file is a part of Julia. License is MIT: http://julialang.org/license
+// This file is a part of Julia. License is MIT: https://julialang.org/license
 
-#ifndef HTABLE_H
-#define HTABLE_H
+#ifndef JL_HTABLE_H
+#define JL_HTABLE_H
 
 #define HT_N_INLINE 32
 
@@ -33,6 +33,14 @@ int HTNAME##_has(htable_t *h, void *key);                       \
 int HTNAME##_remove(htable_t *h, void *key);                    \
 void **HTNAME##_bp(htable_t *h, void *key);
 
+#define HTPROT_R(HTNAME)                                                \
+void *HTNAME##_get_r(htable_t *h, void *key, void *ctx);                \
+void HTNAME##_put_r(htable_t *h, void *key, void *val, void *ctx);      \
+void HTNAME##_adjoin_r(htable_t *h, void *key, void *val, void *ctx);   \
+int HTNAME##_has_r(htable_t *h, void *key, void *ctx);                  \
+int HTNAME##_remove_r(htable_t *h, void *key, void *ctx);               \
+void **HTNAME##_bp_r(htable_t *h, void *key, void *ctx);
+
 #ifdef __cplusplus
 }
 #endif
